/*
 * John Idarraga <idarraga@cern.ch>
 *
 */

#include <iostream>
#include <string>
#include <vector>

#include "TH2.h"
#include "TCanvas.h"
#include "TApplication.h"
#include "TColor.h"
#include "TPaletteAxis.h"
#include "TStyle.h"
#include "TROOT.h"
#include "TPaveLabel.h"
#include "TControlBar.h"
#include "TObject.h"
#include "TString.h"

#include "MediPixViewer.h"
#include "listHandler.h"
#include "control.h"
#include "MediPixWriteToEntuple.h"
#include "allpix_dm.h"

using namespace std;

void checkParameters(int, char**, TString *);
void histoProperties(TH2I *, TCanvas *);
TApplication * g_theApp = new TApplication("Output", 0, NULL);
Int_t g_direction = 0;

int main(int argc, char ** argv){

  /////////////////////////////////////
  // check flags
  TString tempScratchDir = "";
  checkParameters(argc, argv, &tempScratchDir);

  // create ntuple and FramesHandler
  TString dataset = argv[2];
  WriteToNtuple MPXnTuple(dataset, tempScratchDir);
  FramesHandler frames(dataset);

  long int skipFrames = 0;
  if(argc == 5)
	  skipFrames = atoi(argv[4]);

  /////////////////////////////////////
  // Handle the list of files
  ListHandler handlerL(argv[1]);
  std::vector<std::string> listOfFiles = handlerL.getListToLoop(tempScratchDir, FRAME_FILES);
  std::cout << "[INFO] there are " << listOfFiles.size() << " frame files in directory" << std::endl;

  std::vector<std::string> listOfDSCFiles = handlerL.getListToLoop(tempScratchDir, DSC_FILES);
  std::cout << "[INFO] there are " << listOfDSCFiles.size() << " DSC files in directory" << std::endl;


  if(listOfFiles.empty()){
    std::cout << "[ERROR] I could not find any frame files in the specified directory," << std::endl;
    std::cout << "        or the dir does not exist at all ... leaving" << std::endl;
    exit(1);
  }

  /* make sure frames list and DSC list match alright */
  if(listOfDSCFiles.size() != 0)
    listOfDSCFiles = handlerL.OrderFilesPairMatch(listOfFiles, listOfDSCFiles);

/*
  vector<std::string>::iterator itrr = listOfDSCFiles.begin();
  for( ; itrr != listOfDSCFiles.end() ; itrr++){
	  cout << *itrr << endl;
  }
  exit(1);
*/

  if( listOfFiles.size() != listOfDSCFiles.size() ) {
      std::cout << "[ERROR] you have a different amount of dsc and frame files." << std::endl;
      std::cout << "        You probably have an extra file that doesn't correspond" << std::endl;
      std::cout << "        to the data taking ... check it out.  Giving up." << std::endl;
      exit(1);
    }

  Long_t filesItr;
  std::string oneFileName = "";
  std::string oneDSCFileName = "";

  for(filesItr = 0 ; filesItr < (Int_t)listOfFiles.size() ; filesItr++)
    {

	  if(filesItr < skipFrames) continue; // skip if needed

	  oneFileName = listOfFiles[filesItr];
      oneDSCFileName = listOfDSCFiles[filesItr];
      //frames.getAFrameMatrix((TString) oneFileName, (TString) oneDSCFileName);
      bool frameread = frames.readOneFrame((TString) oneFileName, (TString) oneDSCFileName);

      if(frameread) {
    	  MPXnTuple.fillVars(&frames);
      }

    }

  MPXnTuple.closeNtuple();

  std::cout << "[ OK ] done with the conversion.  Check your output file --> " 
	    << MPXnTuple.GetNtupleFileName() << std::endl;


  // erasing possible remaining working files

  std::string command0 = "/bin/bash -c 'if [ -e ";
  if(tempScratchDir.Length() > 0) { command0 += tempScratchDir.Data(); command0 += "/"; }
  command0 += "listOfFiles.txt ] ; then rm -f ";
  if(tempScratchDir.Length() > 0) { command0 += tempScratchDir.Data(); command0 += "/"; }
  command0 += "listOfFiles.txt ; fi'";

  std::string command02 = "/bin/bash -c 'if [ -e ";
  if(tempScratchDir.Length() > 0) { command02 += tempScratchDir.Data(); command02 += "/"; }
  command02 += "listOfFiles.dsc.txt ] ; then rm -f ";
  if(tempScratchDir.Length() > 0) { command02 += tempScratchDir.Data(); command02 += "/"; }
  command02 += "listOfFiles.dsc.txt ; fi'";

  system(command0.c_str());
  system(command02.c_str());

  return 0;
}

void checkParameters(int argc, char ** argv, TString * tempScratchDir){

  if(argc < 3)
    {
      std::cout << "  use:" << std::endl;
      std::cout << "      " << argv[0] << " pathToData  outputFileName  tempScratchDir(optional.  ./ default)  skip(optional)" << std::endl;
      std::cout << "       pathToData   : Directory where your have stored your data in ascii" << std::endl;
      std::cout << "                      format (uncompressed). Can be an absolute or " << std::endl;  
      std::cout << "                      relative path." << std::endl;
      std::cout << "       dataSetNumber: A short string to easily identify your data later on." << std::endl;
      std::cout << "                      Example --> MediPix_SPS_TOTmode_25-06-2007" << std::endl;
      std::cout << "                      please do not use special character or spaces." << std::endl;
      std::cout << "       tempFilesDir : In case you can only read from the directory" << std::endl;
      std::cout << "                      containing the data but can't write, you can" << std::endl;
      std::cout << "                      tell " << argv[0] << "to use a different temp dir."<< std::endl;
      std::cout << "       skip         : Number of events to skip optional (int)" << std::endl;

      exit(1);
    }

  TString dataSet = argv[2];

  if (dataSet == "" || dataSet.Contains(' '))
    {
      std::cout << "[ERROR] badformat in dataSet !" << std::endl;
      std::cout << "        dataSetNumber: example --> MediPix_SPS_TOTmode_25-06-2007" << std::endl;
      std::cout << "        please do not use special character or spaces." << std::endl;
      exit(1);
    }

  // user decided to use a different scratch dir
  // optional
  if(argc > 2){
    *tempScratchDir += argv[3];
    if(tempScratchDir->Length() == 0) *tempScratchDir = "./";
    std::cout << "[INFO] working directory is " << *tempScratchDir << std::endl;
  }

}


void histoProperties(TH2I * h1, TCanvas * c){

  h1->SetStats(false);
  gStyle->SetPalette(1,0); 

  c->SetBorderMode(0);
  c->SetFillColor(kWhite);

  c->Update();

}
